package ca.mcgill.cs.swevo.diffcat.view;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.util.CollectionMap;

public class ClusterManager {

	public static ClusterManager getManager () {
		return manager;
	}
	
	private static final ClusterManager manager;
	static {
		manager = new ClusterManager();
	}
	
	private final Set<IClusterManagerListener> listeners;
	
	private final CollectionMap<String, DiffCatResult> clusters;
	private final CollectionMap<DiffCatResult, String> diffs;
	
	private final CollectionMap<String, String> elementClusters;
	private final CollectionMap<String, String> reverseElements;
	
	private ClusterManager () {
		clusters = new CollectionMap<String, DiffCatResult>();
		diffs = new CollectionMap<DiffCatResult, String>();
		listeners = new HashSet<IClusterManagerListener>();
		elementClusters = new CollectionMap<String, String>();
		reverseElements = new CollectionMap<String, String>();
	}
	
	public void addListener (IClusterManagerListener listener) {
		this.listeners . add (listener);
	}
	
	public void clearData () {
		clusters.clear();
		diffs.clear();
		elementClusters.clear();
		reverseElements.clear();
	}
	
	public void assignToSeed (String seed, DiffCatResult result) {
		clusters.add(seed, result);
		diffs.add(result, seed);
	}
	
	public void assignToSeed (String seed, String element) {
		elementClusters.add(seed, element);
		reverseElements.add(element, seed);
	}
	
	public void assignToSeed (String seed, Collection<DiffCatResult> results) {
		clusters.addAll(seed, results);
		for (DiffCatResult r : results) {
			diffs.add(r, seed);
		}
	}
	
	public void removeFromSeed (String seed, DiffCatResult result) {
		clusters.removeFromKey(seed, result);
		diffs.removeFromKey(result, seed);
		if (clusters.get(seed).isEmpty()) {
			clusters.removeKey(seed);
		}
		if (diffs.get(result).isEmpty()) {
			diffs.removeKey(result);
		}
	}
	
	public void removeFromSeed (String seed, String element) {
		elementClusters.removeFromKey(seed, element);
		if (elementClusters.get(seed).isEmpty()) {
			elementClusters.removeKey(seed);
		}
		reverseElements.removeFromKey(element, seed);
		if (reverseElements.get(element).isEmpty()) {
			reverseElements.removeKey(element);
		}
	}
	
	public Set<String> getSeeds () {
		return Collections.unmodifiableSet(clusters.keySet());
	}
	
	public boolean seedExists (String seed) {
		return clusters.containsKey(seed);
	}
	
	public boolean isPartOfSeed (String seed, DiffCatResult r) {
		return diffs.get(r).contains(seed);
	}
	
	public boolean isPartOfSeed (DiffCatResult r) {
		if (! diffs.containsKey(r)) {
			return false;
		}
		return ! diffs.get(r).isEmpty();
	}
	
	public Set<String> getSeedsFor (DiffCatResult r) {
		return new HashSet<String>(diffs.get(r));
	}
	
	public Set<String> getSeedsFor (String element) {
		return new HashSet<String>(reverseElements.get(element));
	}
	
	public Collection<DiffCatResult> getDiffsFor (String seed) {
		return Collections.unmodifiableCollection(clusters.get(seed));
	}
	
	public Collection<DiffCatResult> getDiffsOutsideOf (String seed) {
		return Collections.unmodifiableCollection(clusters.excludeKey(seed));
	}
	
	public void notifyListeners () {
		for (IClusterManagerListener l : listeners) {
			l.notifySeedChange();
		}
	}
}
