package ca.mcgill.cs.swevo.diffcat.view.actions;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import ca.mcgill.cs.swevo.diffcat.view.ClusterManager;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.view.CategoryView;
import ca.mcgill.cs.swevo.diffcat.view.ClassSigNode;
import ca.mcgill.cs.swevo.diffcat.view.DiffCatNode;
import ca.mcgill.cs.swevo.diffcat.view.FieldSigNode;
import ca.mcgill.cs.swevo.diffcat.view.MethodSigNode;
import ca.mcgill.cs.swevo.diffcat.view.diags.SeedDialog;

public class AssignSeedAction extends DiffCatAction {

	public AssignSeedAction (CategoryView viewHandle) {
		super (viewHandle);
	}
	
	public void run () {
		ISelection selection = view.getTreeSelection();
		IStructuredSelection iStructuredSelection = (IStructuredSelection)selection;
		Object[] array = iStructuredSelection.toArray();
		if (array.length == 0) {
			return;
		}
		String seed = getSeed();
		if (seed == null) {
			return;
		}
		for (Object selected : array) {
			assignSeed (seed, selected);
		}
	}
	
	/**
	 * Utility method. Not conceptually the best place to put this, but 
	 * I don't want to make a new action for this. 
	 */
	public void removeFromSeed (String seed, Object selected) 
	{
		if (selected instanceof DiffCatNode) {
			ClusterManager.getManager().removeFromSeed(seed, ((DiffCatNode) selected).getResult());
		}
		else if (selected instanceof DiffCatResult) {
			ClusterManager.getManager().removeFromSeed(seed, (DiffCatResult) selected);
		}
		else if (selected instanceof MethodSigNode) {
			MethodSigNode sig = (MethodSigNode) selected;
			ClusterManager.getManager().removeFromSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				removeFromSeed (seed, child);
			}
		}
		else if (selected instanceof ClassSigNode) {
			ClassSigNode sig = (ClassSigNode) selected;
			ClusterManager.getManager().removeFromSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				removeFromSeed (seed, child);
			}
		}
		else if (selected instanceof FieldSigNode) {
			FieldSigNode sig = (FieldSigNode) selected;
			ClusterManager.getManager().removeFromSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				removeFromSeed (seed, child);
			}
		}
	}
	
	public void assignSeed (String seed, Object selected) {
		if (selected instanceof DiffCatNode) {
			ClusterManager.getManager().assignToSeed(seed, ((DiffCatNode) selected).getResult());
		}
		else if (selected instanceof DiffCatResult) {
			ClusterManager.getManager().assignToSeed(seed, (DiffCatResult) selected);
		}
		else if (selected instanceof MethodSigNode) {
			MethodSigNode sig = (MethodSigNode) selected;
			ClusterManager.getManager().assignToSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				assignSeed (seed, child);
			}
		}
		else if (selected instanceof ClassSigNode) {
			ClassSigNode sig = (ClassSigNode) selected;
			ClusterManager.getManager().assignToSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				assignSeed (seed, child);
			}
		}
		else if (selected instanceof FieldSigNode) {
			FieldSigNode sig = (FieldSigNode) selected;
			ClusterManager.getManager().assignToSeed(seed, sig.getName());
			for (Object child : sig.getChildren()) {
				assignSeed (seed, child);
			}
		}
	}
	
	private String getSeed () {
		boolean done = false;
		do {
			
			Shell shell = view.getViewSite().getShell();
			SeedDialog d = new SeedDialog(shell);
			int result = d.open();
			
			if (result == Dialog.CANCEL) {
				return null;
			}
			
			String seed = d.getSeed();
			
			if (seed == null || seed.isEmpty()) {
				MessageDialog.openError(shell, "Error", "Must enter a valid seed");
				continue;
			}
			
			ClusterManager manager = ClusterManager.getManager();
			if (! manager.seedExists(seed)) {
				boolean confirm = MessageDialog.openConfirm(shell, "Confirmation", "Seed does not exist. Create seed?");
				if (confirm) {
					return seed;
				}
			}
			else {
				return seed;
			}
			
		} while (! done);
		
		return null;
	}
}
