package ca.mcgill.cs.swevo.diffcat.view.actions;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.view.CategoryView;
import ca.mcgill.cs.swevo.diffcat.view.ClusterManager;
import ca.mcgill.cs.swevo.semdiff.core.Transaction;
import ca.mcgill.cs.swevo.util.CollectionMap;
import ca.mcgill.cs.swevo.util.StringUtil;

public class PrintBenchmarkEntryAction extends DiffCatAction {

	public PrintBenchmarkEntryAction(CategoryView view) {
		super(view);
	}

	@Override
	public void run ()
	{
		ClusterManager manager = ClusterManager.getManager();
		
		Transaction trxn = view.getCurrentTransaction();
		System.out.println(" " + trxn.getId());
		Collection<DiffCatResult> visibleDiffs = view.getVisibleDiffs();
		CollectionMap<String, String> seeds = new CollectionMap<String, String>();
		for (DiffCatResult visibleDiff : visibleDiffs)
		{
			Set<String> seedsFor = manager.getSeedsFor(visibleDiff);

			HashSet<String> sigs = new HashSet<String>();
			if (visibleDiff.getRightMethodSignature() != null) {
				sigs.add(visibleDiff.getRightMethodSignature());
			}
			if (visibleDiff.getLeftMethodSignature() != null) {
				sigs.add(visibleDiff.getLeftMethodSignature());
			}
			if (visibleDiff.getRightFieldSignature() != null) {
				sigs.add(visibleDiff.getRightFieldSignature());
			}
			if (visibleDiff.getLeftFieldSignature() != null) {
				sigs.add(visibleDiff.getLeftFieldSignature());
			}
			
			if (sigs.isEmpty()) {
				if (visibleDiff.getRightClassSignature() != null) {
					sigs.add(visibleDiff.getRightClassSignature());
				}
				if (visibleDiff.getLeftClassSignature() != null) {
					sigs.add(visibleDiff.getLeftClassSignature());
				}
			}
			
			if (! seedsFor.isEmpty()) {
				for (String seedFor : seedsFor) {
					for (String sig : sigs) {
						seeds.add(seedFor, sig);
					}
				}
			}
			else {
				for (String sig : sigs) {
					seeds.add("main task", sig);
				}
			}
		}
		for (String seed : seeds.keySet()) {
			System.out.println("  Task(" + seed + ")");
			Collection<String> sigs = new HashSet<String>(seeds.get(seed));
			for (String sig : sigs) {
				System.out.println("   " + sig);
			}
			System.out.println("  EndTask");
		}
		System.out.println(" EndEntry");
	}

	
}
