package ca.mcgill.cs.swevo.diffcat.view.diff;

import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareEditorInput;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.util.PropertyChangeEvent;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.view.Activator;

public abstract class DiffCatEditorInput extends CompareEditorInput {

	protected File left;
	protected File right;
	protected DiffCatResult result;
	
	public DiffCatEditorInput(CompareConfiguration configuration, File left, File right, DiffCatResult result) {
		super(configuration);
		this.left = left;
		this.right = right;
		this.result = result;
	}

	@Override
	protected Object prepareInput(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		
		try {
			FileNode fNode1 = getFileNodeLeft();
			FileNode fNode2 = getFileNodeRight();
			
			Differencer diff = new DiffCatDifferencer();
			
			Object returnObj = diff.findDifferences(false, new NullProgressMonitor(), null, null, fNode1, fNode2);
			return returnObj;
		} catch (CoreException e) {
			return null;
		}
	}

	@Override
	protected void handleDispose () {
		super.firePropertyChange(new PropertyChangeEvent(this, Activator.DELETED_EVENT, this, this));
		super.handleDispose();
	}
	
	protected abstract FileNode getFileNodeLeft () throws CoreException;
	
	protected abstract FileNode getFileNodeRight () throws CoreException;
	
	public boolean equals (Object o) {
		if (o == null) {
			return false;
		}
		if (o == this) {
			return true;
		}
		if (! o.getClass().equals(getClass())) {
			return false;
		}
		
		DiffCatEditorInput input = (DiffCatEditorInput) o;
		
		return left.compareTo(input.left) == 0 && right.compareTo(input.right) == 0;
	}
}
