package ca.mcgill.cs.swevo.diffcat.view.diff;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;

import org.eclipse.compare.IStreamContentAccessor;
import org.eclipse.compare.ITypedElement;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.graphics.Image;

abstract class FileNode implements IStreamContentAccessor, ITypedElement {

	protected File file;
	
	protected ByteChunk left, middle, right;
	
	public FileNode(File file, int pos, int len, String diff) throws CoreException  {
		super();
		this.file = file;
		initByteChunks(pos, len, diff);
	}
	
	public abstract Object[] getChildren (); 
	
	protected abstract void initByteChunks (int pos, int len, String diff) throws CoreException; 
	
	public InputStream getContents() throws CoreException {
		InputStream is = null;
		try {
			is = new FileInputStream(file);
		} catch(IOException ioe) {
			throw new CoreException(new Status(IStatus.ERROR,"me",ioe.getMessage(),ioe));
		}
		return is;
	}

	public Image getImage() {
		return null;
	}

	public String getName() {
		return file.getName();
	}

	public String getType() {
		String name = file.getName();
		int index = name.lastIndexOf(".");
		
		String type = ITypedElement.UNKNOWN_TYPE;
		if (file.isDirectory()) {
			type = ITypedElement.FOLDER_TYPE;
		} else if (index > -1 && index + 1 < name.length()) {
			type = "txt";
		}
		
		return type;
	}
	
	public boolean equals (Object o) {
		if (! o.getClass().equals(getClass())) {
			return false;
		}
		
		FileNode other = (FileNode) o;
		
		return other.file.compareTo(file) == 0;
	}
	
}