package ca.mcgill.cs.swevo.diffcat.view.filters;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.view.diff.DiffShell;
import ca.mcgill.cs.swevo.util.LRTuple;
import ca.mcgill.cs.swevo.util.op.IOperation;

public class BestGuessShellFinder implements IOperation<DiffShell> {

	protected DiffCatResult result;
	protected DiffShell best = null;
	protected LRTuple<Integer> bestVals;
	
	public BestGuessShellFinder (DiffCatResult r) {
		this.result = r;
	}
	
	@Override
	public DiffShell apply(DiffShell t) {
		if (t == null) {
			return t; // ugh
		}
		if (best == null) {
			best = t;
			bestVals = getIntegers (t);
			return t;
		}
		
		tryDiff (t);
		return t;
	}
	
	public DiffShell getBestGuess () {
		return best;
	}
	
	protected void tryDiff (DiffShell shell) {
		LRTuple<Integer> vals = getIntegers (shell);
		if (result.isDelete()) {
			if (vals.getLeft() < bestVals.getLeft()) {
				best = shell;
				bestVals = vals;
			}
		} else if (result.isInsert()) {
			if (vals.getRight() < bestVals.getRight()) {
				best = shell;
				bestVals = vals;
			}
		} else {
			if (vals.getLeft() < bestVals.getLeft() && 
				vals.getRight() < bestVals.getRight()  ) {
				best = shell;
				bestVals = vals;
			}
		}
	}
	
	protected LRTuple<Integer> getIntegers (DiffShell shell) {
		int left = -1;
		int right = -1;
		if (result.isInsert()) {
			right = Math.abs(shell.rightOff - result.getPosTo()) 
							+ Math.abs(shell.rightOff + shell.rightLen - result.getPosTo() - result.getLenTo());
		} else if (result.isDelete()) {
			left = Math.abs(shell.leftOff - result.getPosFrom())
							+ Math.abs(shell.leftOff + shell.leftLen - result.getPosFrom() - result.getLenFrom());
		} else {
			left = Math.abs(shell.rightOff - result.getPosTo()) 
							+ Math.abs(shell.rightOff + shell.rightLen - result.getPosTo() - result.getLenTo());
			right = Math.abs(shell.leftOff - result.getPosFrom())
							+ Math.abs(shell.leftOff + shell.leftLen - result.getPosFrom() - result.getLenFrom());
		}
		
		return new LRTuple<Integer>(left, right);
	}
}
