/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.view.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.ScaleFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ca.mcgill.cs.swevo.diffcat.view.Activator;

public class EdgeTypeWeightPage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	public EdgeTypeWeightPage() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("Select edge weights");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		addField(
			new ScaleFieldEditor(
					PreferenceConstants.P_CONTAINMENT_EDGE_WEIGHT,
					"Select containment edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
			new ScaleFieldEditor(
					PreferenceConstants.P_DIRECT_EDGE_WEIGHT,
					"Select direct edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_HIERARCHY_EDGE_WEIGHT,
					"Select hierarchy edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_STRUCT_SIM_EDGE_WEIGHT,
					"Select structural similarity edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));

		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_REFERENCE_EDGE_WEIGHT,
					"Select shared method call edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_VARIABLE_EDGE_WEIGHT,
					"Select shared variable usage edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_SOURCE_EDGE_WEIGHT,
					"Select shared variable dependency edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_SINK_EDGE_WEIGHT,
					"Select shared variable closure edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_CLONE_EDGE_WEIGHT,
					"Select clone edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
		addField(
				new ScaleFieldEditor(
					PreferenceConstants.P_SIG_EDGE_WEIGHT,
					"Select signature edge weight",
					getFieldEditorParent(), 0, 5, 1, 1));
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

}
