/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CatchClause;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;


public class CatchClauseFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final String[] caughtException;
	private boolean stopVisiting;
	
	public CatchClauseFinder (SourceCodeEntity catchClause, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.caughtException = CDUtil.getTypeFQN(catchClause);
		this.stopVisiting = false;
	}
	
	public boolean visit (CatchClause catchClause) {
		
		if (stopVisiting) {
			return false;
		}
		
		if (! inCorrectMethodDecl(catchClause, methodDecl)) {
			return true;
		}
		
		SingleVariableDeclaration svd = catchClause.getException();
		String[] typeFQN = ASTUtil.getTypeFQNViaStrings(svd);
		
		if (super.fullyOrPartialMatches(typeFQN, caughtException)) {
			super.assocNode = svd;
			stopVisiting = true;
			return false;
		}
		
		return true;
	}
	
}
