/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.DoStatement;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.WhileStatement;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class ConditionFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final String statement;
	private boolean stopVisiting;
	
	public ConditionFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.statement = statement.getUniqueName();
		this.stopVisiting = false;
	}
	
	public boolean visit (ForStatement s) {
		return visitStatementExpr(s, s.getExpression());
	}
	
	public boolean visit (IfStatement s) {
		return visitStatementExpr(s, s.getExpression());
	}
	
	public boolean visit (WhileStatement s) {
		return visitStatementExpr(s, s.getExpression());
	}
	
	public boolean visit (EnhancedForStatement s) {
		
		// do it like this because CD says the whole thing was updated
		
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(s, methodDecl)) {
			return false;
		}
		
		if (! this.statement.contains(s.getParameter().toString())) {
			return true;
		}
		
		if (! this.statement.contains(s.getExpression().toString())) {
			return true;
		}
		
		super.assocNode = s;
		this.stopVisiting = true;
		
		return false;
	}
	
	public boolean visit (DoStatement doStatement) {
		return visitStatementExpr (doStatement, doStatement.getExpression());
	}
	
	private boolean visitStatementExpr (Statement statement, Expression expression)
	{
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		String statementString = expression.toString().trim();
		if (statementString.equals(this.statement)) {
			super.assocNode = expression;
			this.stopVisiting = true;
		}
		
		return true;
	}
}
