/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.IfStatement;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class ElseBlockFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final String statement;
	private boolean stopVisiting;
	
	public ElseBlockFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.statement = statement.getUniqueName();
		this.stopVisiting = false;
	}
	
	public boolean visit (IfStatement statement)
	{
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		String statementString = statement.getExpression().toString().trim();
		if (statementString.equals(this.statement)) {
			super.assocNode = statement.getElseStatement();
			this.stopVisiting = true;
		}
		
		return true;
	}
	
}
