/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;
import org.evolizer.changedistiller.model.entities.StructureEntityVersion;

import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;


public class FieldDeclarationFinder extends ANodeFinder  {

	private String   declName;	// don't need more than this to find fields
	
	public FieldDeclarationFinder (StructureEntityVersion rootEntity) {
		this.declName = CDUtil.getFieldName(rootEntity);
	}
	
	public FieldDeclarationFinder (SourceCodeEntity fieldDecl) {
		this.declName = CDUtil.getFieldName(fieldDecl);
	}
	
	public FieldDeclarationFinder (String fieldName) {
		this.declName = fieldName;
	}
	
	public boolean visit (FieldDeclaration declaration)
	{
		String declName = ASTUtil.getFieldDeclName(declaration);
		
		if (! declName.equals(this.declName)) {
			
			// check if has multiple field names
			if (declaration.fragments().size() > 1) {
				for (int i = 0; i < declaration.fragments().size(); i++) {
					VariableDeclarationFragment fragment = (VariableDeclarationFragment) declaration.fragments().get(i);
					if (this.declName.equals(fragment.getName().toString())) {
						super.assocNode = declaration;
						return false;
					}
				}
			}
			
			return true;
		}
		
		super.assocNode = declaration;
		return false;
	}
}
