/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import java.util.List;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ForStatement;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class ForStatementFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private boolean stopVisiting;
	private final String statement;
	
	public ForStatementFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.statement = statement.getUniqueName();
		this.stopVisiting = false;
	}
	
	@SuppressWarnings("unchecked")
	public boolean visit (ForStatement statement) {
		
		if (stopVisiting) {
			return false;
		}
		
		if (! inCorrectMethodDecl(statement, methodDecl)) {
			return false;
		}
		
		List initializers = statement.initializers();
		
		for (int i = 0; i < initializers.size(); i++) {
			Object initializer = initializers.get(i);
			if (initializer.toString().equals(this.statement)) {
				stopVisiting = true;
				assocNode = (ASTNode) initializer;
			}
		}
		
		List updaters = statement.updaters();
		
		for (int i = 0; i < updaters.size(); i++) {
			Object updater = updaters.get(i);
			if (updater.toString().equals(this.statement)) {
				stopVisiting = true;
				assocNode = (ASTNode) updater;
				return false;
			}
		}
		
		Expression expression = statement.getExpression();
		
		if (expression.toString().equals(this.statement)) {
			stopVisiting = true;
			assocNode = expression;
			return false;
		}
		
		return true;
	}
	
}
