/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;
import org.evolizer.changedistiller.model.entities.StructureEntityVersion;

import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;


public class SoftMethodDeclarationFinder extends ANodeFinder {

	private String[] declTypeFQN;
	private String   declName;
	private String[][] declTypeArgs;
	
	public SoftMethodDeclarationFinder (SourceCodeEntity methodDeclClass, SourceCodeEntity methodDeclName, IMethodBinding newBinding)
	{
		this.declTypeFQN 	= CDUtil.getTypeFQN(methodDeclClass);
		this.declName 		= CDUtil.getMethodName(methodDeclName);
		this.declTypeArgs	= BindingUtil.getParamTypes(newBinding);
	}
	
	/**
	 * Initialize a new MethodDeclarationFinder
	 * @param methodDecl METHOD_DECLARATION of the form fqn.name(arg1,...,arg2)
	 */
	public SoftMethodDeclarationFinder (SourceCodeEntity methodDecl)
	{
		this.declTypeFQN 	= CDUtil.getMethodDeclaringType(methodDecl);
		this.declName 		= CDUtil.getMethodName(methodDecl);
		this.declTypeArgs	= CDUtil.getMethodParameterTypesIgnoreBrackets(methodDecl);
	}
	
	/**
	 * Initialize a new MethodDeclarationFinder
	 * @param methodDecl METHOD_DECLARATION of the form fqn.name(arg1,...,arg2)
	 */
	public SoftMethodDeclarationFinder (StructureEntityVersion methodDecl)
	{
		this.declTypeFQN 	= CDUtil.getMethodDeclaringType(methodDecl);
		this.declName 		= CDUtil.getMethodName(methodDecl);
		this.declTypeArgs	= CDUtil.getMethodParameterTypesIgnoreBrackets(methodDecl);
	}
		
	public boolean visit (MethodDeclaration node) {
		IMethodBinding binding = node.resolveBinding();
		
		String[] declTypeFQN = null;
		String 	 declName = null;
		String[][] declTypeArgs = null;
		
		if (binding != null) {
			declTypeFQN = BindingUtil.getDeclaringType(binding);
			declName    = BindingUtil.getMethodName(binding);
			declTypeArgs = BindingUtil.getParamTypesIgnoreBrackets(binding);
		}
		else {
			
			ASTNode parent = node.getParent();
			
			if (parent instanceof AbstractTypeDeclaration) {
				AbstractTypeDeclaration decl = (AbstractTypeDeclaration) parent;
				ITypeBinding binding2 = decl.resolveBinding();
				
				if (binding2 == null) {
					declTypeFQN = this.declTypeFQN;
				}
				else {
					declTypeFQN = BindingUtil.getTypeFQN(binding2);
				}
				
			}
			else  {
				return false;	// inner method. assume change distiller did not detect
			}
			
			declName = ASTUtil.getMethodName(node);
			declTypeArgs = ASTUtil.getParamTypesIgnoreBrackets(node);
		}
		 
		
		if (! declName.equals(this.declName)) {
			return true;
		}
		
		if (! this.declTypeFQN[0].equals("hackCLASS")) {
			if (! super.fullyOrPartialMatches(this.declTypeFQN, declTypeFQN)) {
				return true;
			}
		}
		
		if (! super.partialMatches(this.declTypeArgs, declTypeArgs)) {
			return true;
		}
		
		super.assocNode = node;
		return false;
			
	}
	
}
