/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import java.util.Collection;

import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.evolizer.changedistiller.model.classifiers.ChangeType;
import org.evolizer.changedistiller.model.entities.SourceCodeChange;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;
import org.evolizer.changedistiller.model.entities.Update;

import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;


public class UltraFieldDeclarationFinder extends ANodeFinder {

	private String[] oldFieldType;
	private String oldFieldName;
	
	private final String[] newFieldType;
	private final String newFieldName;
	
	public UltraFieldDeclarationFinder (FieldDeclaration right, Collection<SourceCodeChange> changes)
	{
		this.newFieldName = ASTUtil.getFieldDeclName(right);
		this.newFieldType = BindingUtil.getTypeFQN(right.getType().resolveBinding());
		
		for (SourceCodeChange change : changes) {
			if (change.getChangeType().equals(ChangeType.ATTRIBUTE_RENAMING)) {
				Update update = (Update) change;
				SourceCodeEntity entity = update.getChangedEntity();
				this.oldFieldName = CDUtil.getFieldName(entity);
			}
			if (change.getChangeType().equals(ChangeType.ATTRIBUTE_TYPE_CHANGE)) {
				Update update = (Update) change;
				SourceCodeEntity entity = update.getChangedEntity();
				this.oldFieldType = CDUtil.getTypeFQN(entity);
			}
		}
		
		if (this.oldFieldName == null) {
			this.oldFieldName = this.newFieldName;
		}
		
		if (this.oldFieldType == null) {
			this.oldFieldType = this.newFieldType;
		}
	}
	
	public boolean visit (FieldDeclaration declaration) 
	{
		String[] type = getType(declaration);
		
		if (! super.fullyOrPartialMatches(type, this.oldFieldType)) {
			String clazz = type[type.length - 1];
			if (clazz.endsWith("[]")) {
				type[type.length - 1] = clazz.substring(0, clazz.length() - 2);
				if (! super.fullyOrPartialMatches(type, this.oldFieldType)) {
					return false;
				}
			}
		}
		
		String name = ASTUtil.getFieldDeclName(declaration);
		
		if (! name.equals(oldFieldName)) {
			return false;
		}
		
		super.assocNode = declaration;
		
		return false;
	}
}
