/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.reconciler;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.core.dom.SimpleName;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.ast.VariableUsageCollector;

public class VarExtractionReconciler extends AbstractReconciler {

	@Override
	public Set<DiffCatResult> reconcile(Set<DiffCatResult> codeChanges) {
		
		for (DiffCatResult codeChange : new HashSet<DiffCatResult>(codeChanges)) {
			if (codeChange.getChangeType().equals(ChangeOperation.AFFECTED_BY_VAR_EXTRACTION.name())) {
				
				Set<DiffCatResult> affecting = getAssociatedVarExtracts(codeChange, codeChanges);
				
				String nodeFromString = codeChange.getNodeFrom().toString();
				String nodeToString = constructNodeToGivenExtracts(codeChange, affecting);
				
				if (! nodeFromString.equals(nodeToString)) {
					codeChange.setChangeType(ChangeOperation.STATEMENT_UPDATE.name());
				}
			}
		}
		
		Set<DiffCatResult> copy = new HashSet<DiffCatResult>(codeChanges);
		for (DiffCatResult codeChange : codeChanges) {
			if (codeChange.getChangeType().equals(ChangeOperation.LOCAL_VARIABLE_EXTRACTION_INSERT.name())) {
				
				SimpleName name = getFragment(codeChange.getNodeTo()).getName();
				VariableUsageCollector col = new VariableUsageCollector(name);
				
				boolean hasAffectedStmt = false;
				for (DiffCatResult c : copy) {
					if (c.getChangeType().equals(ChangeOperation.AFFECTED_BY_VAR_EXTRACTION.name())) {
						c.getNodeTo().accept(col);
						hasAffectedStmt |= col.getOccurrences().size() == 1;
						col.clear();
					}
				}
				
				if (! hasAffectedStmt) {
					codeChange.setChangeType(ChangeOperation.VARIABLE_DECLARATION_STATEMENT_INSERT.name());
				}
			}
		}
		
		return codeChanges;
	}
}
