/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.refiner;

import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.ReturnStatement;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsReturnStmtInsDelFilter;

public class ReturnStatementRefiner extends AbstractRefiner {

	private final IsReturnStmtInsDelFilter filter = new IsReturnStmtInsDelFilter();
	
	@Override
	public Set<DiffCatResult> refine(DiffCatResult result) {
		
		if (! filter.passesFilter(result)) {
			return singleton(result);
		}
		
		ReturnStatement rtrnStmt = null;
		MethodDeclaration parentMethod = null;
		MethodDeclaration otherParent = null;
		
		if (result.isDelete()) 
		{
			rtrnStmt = (ReturnStatement) result.getNodeFrom();
			parentMethod = (MethodDeclaration) result.getLeftParent();
			otherParent = (MethodDeclaration) result.getRightParent();
		}
		
		else
		{
			rtrnStmt = (ReturnStatement) result.getNodeTo();
			parentMethod = (MethodDeclaration) result.getRightParent();
			otherParent = (MethodDeclaration) result.getLeftParent();
		}
		
		if (rtrnStmt == null) {
			return singleton(result);
		}
		
		if (rtrnStmt.getExpression() != null) {
			if (! rtrnStmt.getExpression().toString().isEmpty()) {
				return singleton(result);
			}
		}
		
		if(! ASTUtil.isLastReturnStatement(parentMethod, rtrnStmt)) {
			return singleton(result);
		}

		if (rtrnStmt.getParent().getNodeType() != ASTNode.METHOD_DECLARATION) {
			return singleton(result);
		}
		
		if (parentMethod.getReturnType2() != null) {
			if (! parentMethod.getReturnType2().toString().equals("void")) {
				return singleton(result);
			}
		}
		
		if (otherParent.getReturnType2() != null) {
			if (! otherParent.getReturnType2().toString().equals("void")) {
				return singleton(result);
			}
		}
		
		result.setChangeType(result.isDelete() ? ChangeOperation.TRIVIAL_RETURN_STATEMENT_DELETE.name() :
								ChangeOperation.TRIVIAL_RETURN_STATEMENT_INSERT.name());
		
		return singleton(result);
	}

}
