/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.refiner;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jface.viewers.IBaseLabelProvider;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.reconciler.AbstractReconciler;
import ca.mcgill.cs.swevo.diffcat.util.DiffCatParentMap;
import ca.mcgill.cs.swevo.diffcat.util.ast.FieldAccessCollector;
import ca.mcgill.cs.swevo.diffcat.util.ast.VariableUsageCollector;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsRenameFilter;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsVarExtractionFilter;
import ca.mcgill.cs.swevo.util.SetUtil;

/**
 * Identifies trivial 'this' insertions and deletions. Can find
 * cases where multiple this expressions were inserted and/or deleted. 
 * 
 * e.g., Ant 1578
 * 
 * @author david
 *
 */
public class ThisExpressionRefiner extends AbstractRefiner {

//	@Override
//	public Set<DiffCatResult> reconcile (Set<DiffCatResult> results) {
//		
//		Set<DiffCatResult> newResults = new HashSet<DiffCatResult>();
//		Set<DiffCatResult> extractions = SetUtil.filterAndCopy(results, new IsVarExtractionFilter());
//		
//		for (DiffCatResult result : results) 
//		{
//			newResults.addAll(refine(result, extractions));
//		}
//		
//		return newResults;
//	}
	
	public Set<DiffCatResult> refine(DiffCatResult result) {
		
		// TODO incorporate var extraction knowledge
		
		if (! result.isStatementUpdate()) {
			return singleton(result);
		}
		
		// TODO Nutch trxn #160 (can't fix this because CD misses type p[] brackets  
		if (result.getNodeFrom() == null || result.getNodeTo() == null) {
			return singleton(result);
		}
		
		WHAT what = sameModuloThisKeywords(result);
		if (what == WHAT.NONE)
			return singleton(result);
		else {
			if (what == WHAT.INSERT)
				result.setChangeType(ChangeOperation.TRIVIAL_THIS_EXPRESSION_INSERT.name());
			else 
				result.setChangeType(ChangeOperation.TRIVIAL_THIS_EXPRESSION_DELETE.name());
		}
		
		return singleton(result);
		
	}
	
	private WHAT sameModuloThisKeywords (DiffCatResult result)
	{
		List<ASTNode> fromFlat = flatten(result.getNodeFrom());
		List<ASTNode> toFlat   = flatten(result.getNodeTo());
		
		int fromCounter = 0;
		int toCounter = 0;
		
		boolean sawThisDelete = false;
		boolean sawThisInsert = false;
		while (fromCounter < fromFlat.size() && toCounter < toFlat.size())
		{
			ASTNode subOne = fromFlat.get(fromCounter);
			ASTNode subTwo = toFlat.get(toCounter);
			
			if (subOne.getNodeType() == subTwo.getNodeType()) {
				if(! areSame(subOne, subTwo)) {
					return WHAT.NONE;
				}
				
				toCounter ++;
				fromCounter ++;
				
				continue;
			}
			else if (subOne.getNodeType() == ASTNode.THIS_EXPRESSION) {
				fromCounter += 1;
				sawThisDelete = true;
			}
			else if (subOne.getNodeType() == ASTNode.FIELD_ACCESS && subTwo.getNodeType() == ASTNode.SIMPLE_NAME) {
				FieldAccess f1 = (FieldAccess) subOne;
				SimpleName f2 = (SimpleName) subTwo;
				
				if (! areEqualBindings(f1.getName(), f2)) {
					return WHAT.NONE;
				}
				
				fromCounter ++;
			}
			else if (subTwo.getNodeType() == ASTNode.THIS_EXPRESSION) {
				toCounter += 1;
				sawThisInsert = true;
			}
			else if (subOne.getNodeType() == ASTNode.SIMPLE_NAME && subTwo.getNodeType() == ASTNode.FIELD_ACCESS) {
				FieldAccess f1 = (FieldAccess) subTwo;
				SimpleName f2 = (SimpleName) subOne;
				
				if (! areEqualBindings(f1.getName(), f2)) {
					return WHAT.NONE;
				}
				
				toCounter ++;
			}
			else {
				return WHAT.NONE;
			}
		}
		
		if (fromCounter != fromFlat.size() || toCounter != toFlat.size()) {
			return WHAT.NONE;
		}
		
		if (sawThisDelete &&sawThisInsert) {
			return WHAT.BOTH;
		}
		else if (sawThisDelete) {
			return WHAT.DELETE;
		}
		else {
			return WHAT.INSERT;
		}
	}
	
	private enum WHAT {
		DELETE,
		INSERT,
		BOTH,
		NONE;
	}
}
