/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.refiner;

import java.util.Collections;
import java.util.Set;

import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.Type;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;

public class TypeUpdateRefiner extends AbstractRefiner {

	public boolean canProcess(DiffCatResult result) {
		return result.getChangeType().equals(ChangeOperation.RETURN_TYPE_UPDATE.name()) ||
			   result.getChangeType().equals(ChangeOperation.ATTRIBUTE_TYPE_UPDATE.name()) || 
			   result.getChangeType().equals(ChangeOperation.PARAMETER_TYPE_UPDATE.name()) ||
			   result.getChangeType().equals(ChangeOperation.VARIABLE_DECLARED_TYPE_UPDATE.name());
	}

	@Override
	public Set<DiffCatResult> refine(DiffCatResult result) {
		
		if (! canProcess(result)) {
			return singleton(result);
		}
		
		Type leftReturnType = (Type) result.getNodeFrom();
		Type rightReturnType = (Type) result.getNodeTo();
		
		ITypeBinding leftBinding = leftReturnType.resolveBinding();
		ITypeBinding rightBinding = rightReturnType.resolveBinding();
		
		if (BindingUtil.areEqual(leftBinding, rightBinding)) {
			String changeType = result.getChangeType();
			if (changeType.equals(ChangeOperation.RETURN_TYPE_UPDATE.name())) {
				result.setChangeType(ChangeOperation.RETURN_TYPE_TRIVIAL_UPDATE.name());
			} else if (changeType.equals(ChangeOperation.ATTRIBUTE_TYPE_UPDATE.name())) {
				result.setChangeType(ChangeOperation.ATTRIBUTE_TYPE_UPDATE.name());
			} else if (changeType.equals(ChangeOperation.PARAMETER_TYPE_UPDATE.name())) {
				result.setChangeType(ChangeOperation.PARAMETER_TYPE_TRIVIAL_UPDATE.name());
			} else if (changeType.equals(ChangeOperation.VARIABLE_DECLARED_TYPE_UPDATE.name())) {
				result.setChangeType(ChangeOperation.VARIABLE_DECLARED_TYPE_TRIVIAL_UPDATE.name());
			} 
		}
		
		return Collections.singleton(result);
	}
}
