/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.ast;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.InfixExpression;

public class ASTFlattener extends ASTVisitor {

	private List<ASTNode> flattened;
	
	public ASTFlattener () {
		flattened = new ArrayList<ASTNode>();
	}
	   
	@Override
	public void preVisit (ASTNode node) {
		flattened.add(node);
	}
	
	@Override
	public boolean visit (InfixExpression expression) {
		flattened.add(expression);
		if (expression.getLeftOperand() != null) {
			expression.getLeftOperand().accept(this);
		}
		if (expression.getRightOperand() != null) {
			expression.getRightOperand().accept(this);
		}
		for (Object o : expression.extendedOperands()) {
			Expression e = (Expression) o;
			e.accept(this);
		}
		
		return false;
	}
	
	public List<ASTNode> getFlattenedAndClear () {
		List<ASTNode> flat = new ArrayList<ASTNode>(flattened);
		flattened.clear();
		return flat;
	}
}
