/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.filter;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.util.filter.IFilter;

public class IsChangeOperationFilter implements IFilter<DiffCatResult> {
	
	private Set<String> ops;
	
	public IsChangeOperationFilter (String op) {
		this.ops = Collections.singleton(op);
	}
	
	public IsChangeOperationFilter (String...ops) {
		this.ops = new HashSet<String>();
		for (String op : ops) {
			this.ops.add(op);
		}
	}
	
	public IsChangeOperationFilter (Set<String> ops) {
		this.ops = new HashSet<String>(ops);
	}
	
	@Override
	public boolean passesFilter(DiffCatResult t) {
		return ops.contains(t.getChangeType());
	}

	public Set<String> getOps () {
		return ops;
	}
}
