/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import ca.mcgill.cs.swevo.util.filter.IFilter;

public class LRTupleMap <K, V> {

	private Map<LRTuple<K>, V> map;
	
	public LRTupleMap () {
		map = new HashMap<LRTuple<K>, V>();
	}
	
	public void put (K left, K right, V val) {
		map.put(new LRTuple<K>(left, right), val);
	}
	
	public void put (LRTuple<K> key, V val) {
		map.put(key, val);
	}
	
	public V get (K left, K right) {
		return map.get(new LRTuple<K>(left, right));
	}
	
	public V get (LRTuple<K> pair) {
		return map.get(pair);
	}
	
	public void remove (K left, K right) {
		map.remove(new LRTuple<K>(left, right));
	}
	
	public Set<LRTuple<K>> keySet () {
		return map.keySet();
	}
	
	public void clear () {
		map.clear();
	}
	
	public LRTupleMap<K, V> getAllInvolvingLeft (K key) {
		LRTupleMap<K, V> subMap = new LRTupleMap<K, V>();
		for (LRTuple<K> keyPair : map.keySet()) {
			if (keyPair.getLeft().equals(key)) {
				subMap.put(keyPair, map.get(keyPair));
			}
		}
		return subMap;
	}
	
	public LRTupleMap<K, V> getAllInvolvingRight (K key) {
		LRTupleMap<K, V> subMap = new LRTupleMap<K, V>();
		for (LRTuple<K> keyPair : map.keySet()) {
			if (keyPair.getRight().equals(key)) {
				subMap.put(keyPair, map.get(keyPair));
			}
		}
		return subMap;
	}
}
