/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import ca.mcgill.cs.swevo.util.filter.IFilter;

public class LRTupleSet <T> implements Iterable<LRTuple<T>> {

	private Set<LRTuple<T>> set;
	
	public LRTupleSet () {
		set = new HashSet<LRTuple<T>>();
	}
	
	public LRTupleSet (Set<LRTuple<T>> set) {
		this.set = new HashSet<LRTuple<T>>(set);
	}
	
	public void add (LRTuple<T> tuple) {
		set.add(tuple);
	}
	
	public void add (T left, T right) {
		add(new LRTuple<T>(left, right));
	}
	
	public void retainAll (IFilter<LRTuple<T>> filter) {
		Collection<LRTuple<T>> retain = new LinkedList<LRTuple<T>>();
		for (LRTuple<T> tuple : this) {
			if (filter.passesFilter(tuple)) {
				retain.add(tuple);
			}
		}
		this.set.retainAll(retain);
	}
	
	@Override
	public Iterator<LRTuple<T>> iterator() {
		return set.iterator();
	}
	
	public int getSize () {
		return set.size();
	}
	
	public void removeSameLeftRightTuples () {
		retainAll(new DifferentLeftRightFilter());
	}
	
	private class DifferentLeftRightFilter implements IFilter<LRTuple<T>> {

		@Override
		public boolean passesFilter(LRTuple<T> t) {
			if (t.getLeft() == t.getRight()) {
				return false;
			}
			else if (t.getLeft() != null) {
				return ! t.getLeft().equals(t.getRight());
			}
			else {
				return true;
			}
		}
		
	}
}
