/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ca.mcgill.cs.swevo.util.filter.IFilter;

public class MapUtil {

	public static <T, F> Map<T, F> transpose (Map<F, T> map) {
		Map<T, F> transpose = new HashMap<T, F>();
		for (F key : map.keySet()) {
			T val = map.get(key);
			transpose.put(val, key);
		}
		return transpose;
	}

	public static <T, V> CollectionMap<T, V> transform (Map<T, V> map) {
		CollectionMap<T, V> collectionMap = new CollectionMap<T, V>();
		for (T item : map.keySet()) {
			collectionMap.add(item, map.get(item));
		}
		return collectionMap;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	/**
	 * Removes keys from the map based on their values. 
	 */
	public static <V> void filterMapOnValues (Map<?, V> map, IFilter<V> filter) {
		HashSet keys = new HashSet(map.keySet());
		for (Object key : keys) {
			V value = map.get(key);
			if (! filter.passesFilter(value)) {
				map.remove(key);
			}
		}
	}

	/**
	 * Returns a copy of the map's keyset, to enable in-loop modification
	 * of the map.
	 */
	public static <K> Set<K> copyKeySet (Map<K, ?> map) {
		return new HashSet<K>(map.keySet());
	}
	
	public static <K> Iterable<K> iterateKeysSorted (Map<K, ?> map, Comparator<K> com)
	{
		List<K> keys = new ArrayList<K>(map.keySet());
		Collections.sort(keys, com);
		return keys;
	}
	
	public static <K extends Comparable<K>> List<K> getKeysSorted (Map<K, ?> map) {
		List<K> keys = new ArrayList<K>(map.keySet());
		Collections.sort(keys);
		return keys;
	}
	
	public static <T extends Comparable<T>> Iterable<T> iterateKeysSorted (Map<T, ?> map)
	{
		List<T> keys = new ArrayList<T>(map.keySet());
		Collections.sort(keys); 
		return keys;
	}
	
	public static <K, V extends Comparable<V>> Iterable<K> iterateKeysSortedByValue (final Map<K, V> map)
	{
		List<K> keys = new ArrayList<K>(map.keySet());
		Collections.sort(keys, new Comparator<K>() {
			public int compare(K key0, K key1) {
				return map.get(key0).compareTo(map.get(key1));
			}
		});
		return keys;
	}
	
	public static <K, V> Iterable<K> iterateKeysSortedByValue (final Map<K, V> map, final Comparator<V> comparator)
	{
		List<K> keys = new ArrayList<K>(map.keySet());
		Collections.sort(keys, new Comparator<K>() {
			public int compare(K key0, K key1) {
				return comparator.compare(map.get(key0), map.get(key1));
			}
		});
		return keys;
	}
}
