/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.util.filter.IFilter;
import ca.mcgill.cs.swevo.util.op.IConverter;
import ca.mcgill.cs.swevo.util.op.IOperation;

public class SetUtil {

	public static <T> Set<T> apply (Set<T> set, IOperation<T> op) {
		Set<T> newSet = new HashSet<T>();
		for (T t : set) {
			t = op.apply(t);
			if (t!=null) {
				newSet.add(t);
			}
		}
		return newSet;
	}
	
	public static <T> Set<T> filterAndCopy (Set<T> set, IFilter<T> filter) {
		Set<T> retain = new HashSet<T>();
		for (T item : set) {
			boolean passes = filter.passesFilter(item);
			if (passes) {
				retain.add(item);
			}
		}
		return retain;
	}

	public static <T> Set<T> filterAndCopy (Set<T> set, Class<? extends T> c) {
		Set<T> retain = new HashSet<T>();
		for (T item : set) {
			boolean passes = (item.getClass() == c);
			if (passes) {
				retain.add(item);
			}
		}
		return retain;
	}
	
	public static <T> void removeAll (Set<T> set, Class<? extends T> c) {
		set.removeAll(filterAndCopy(set, c));
	}
	
	public static <T> Set<T> removeAll (Set<T> set, IFilter<T> f) {
		set.removeAll(filterAndCopy(set, f));
		return set;
	}
	
	@SuppressWarnings("unchecked")
	public static <T> LRTupleSet<T> getUnorderedPairs (Set<T> set) {
		Object[] array = set.toArray();
		LRTupleSet<T> pairs = new LRTupleSet<T>();
		for (int i = 0; i < array.length; i++) {
			for (int j = i+1; j < array.length; j++) {
				pairs.add((T) array[i], (T) array[j]);
			}
		}
		
		return pairs;
	}

	@SuppressWarnings("unchecked")
	public static <T extends Comparable<T>> LRTupleSet<T> getOrderedPairs (Set<T> set) {
		Object[] array = set.toArray();
		LRTupleSet<T> pairs = new LRTupleSet<T>();
		for (int i = 0; i < array.length; i++) {
			for (int j = i+1; j < array.length; j++) {
				
				T left = (T) array[i];
				T right = (T) array[j];
				
				if (right.compareTo(left) < 0) {
					T mid = right;
					right = left;
					left = mid;
				}
				
				pairs.add(left, right);
			}
		}
		
		return pairs;
	}
	
	public static <T> Set<T> intersection (Collection<T> a, Collection<T> b) {
		Set<T> set = new HashSet<T>();
		intersection (a, b, set);
		return set;
	}
	
	public static <T> void intersection (Collection<T> a, Collection<T> b, Set<T> set)
	{
		set.addAll(a);
		for (T t : a) {
			if (! b.contains(t)) {
				set.remove(t);
			}
		}
	}
	
	public static <T> Set<T> intersection (T[] a, T[] b) {
		return intersection (Arrays.asList(a), Arrays.asList(b));
	}
	
	public static <T> Set<T> union (Collection<T> a, Collection<T> b) {
		Set<T> set = new HashSet<T>(a);
		set.addAll(b);
		return set;
	}
	
	public static <F, T> Set<T> convert (Set<F> col, IConverter<F, T> converter) {
		Set<T> newCol = new HashSet<T>();
		for (F el : col) {
			newCol.add(converter.convert(el));
		}
		return newCol;
	}
	
	public static <T> void transfer (Set<T> from, Set<T> to) {
		to.addAll(from);
		from.clear();
	}
	
	public static <T> int minSetSize (Set<T> a, Set<T> b) {
		return Math.min(a.size(), b.size()); 
	}
}
