/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

public class StringUtil {

	public static final String newLine = System.getProperty("line.separator");
	
	public static String eliminateDocs (String s) {
		return s.replaceAll("/\\\\*.*/\\\\*", "");
	}
	
	public static String eliminateComments (String s) {
		return s.replaceAll("//.*"+newLine, "");
	}
	
	public static int findFirstCurlyBrace (String s) {
		return s.indexOf('{');
	}
	
	public static int findFirstEqualsSign (String s) {
		return s.indexOf('=');
	}
	
	public static String findMethodSignature (String s) {
		int index = findFirstCurlyBrace(s);
		if (index != -1) {
			return s.substring(0, index).trim();
		}
		return s;
	}
	
	public static String findFieldSignature (String s) {
		s = s.replace(';', ' ');
		int index = findFirstEqualsSign(s);
		if (index == -1) {
			return s.trim();
		}
		return s.substring(0, index).trim();
	}
	
	public static String normalizeWhitespace (String s) {
		return s.replaceAll("\\p{javaWhitespace}{2,}", " ");
	}
	
	public static String eliminateTypeArgs (String args) {
		while (args.contains("<")) {
			int lIndex = args.indexOf('<');
			int numSeen = 1;
			for (int rIndex = lIndex + 1; rIndex < args.length(); rIndex++) {
				char c = args.charAt(rIndex);
				if (c == '<') {
					numSeen++;
				}
				else if (c == '>') {
					numSeen--;
				}
				
				if (numSeen == 0) {
					args = args.substring(0, lIndex) + args.substring(rIndex + 1);
					break;
				}
			}
		}
		
		return args;
	}
	
	public static String normalizeType (String type) {
		if (type.contains(".")) {
			StringBuilder b = new StringBuilder();
			String[] parts = type.split("\\.");
			boolean first = true;
			for (int i = 0; i < parts.length; i++) {
				String part = parts[i];
				if (Character.isUpperCase(part.charAt(0))) {
					if (first) {
						first = false;
					} else {
						b.append(".");
					}
					b.append(part);
				}
			}
			return b.toString();
		}
		else {
			return type;
		}
	}
	
	public static String getShortHandFieldSig (String fieldBody) 
	{
		fieldBody = eliminateDocs (fieldBody);
		fieldBody = eliminateComments (fieldBody);
		
		String fieldSig = findFieldSignature(fieldBody);
		
		fieldSig = normalizeWhitespace(fieldSig).trim();
		
		String[] parts = fieldSig.split(" ");
		
		return parts[parts.length-1];
	}
	
	public static String getShortHandTypeSig (String typeBody)
	{
		typeBody = eliminateDocs(typeBody);
		typeBody = eliminateComments(typeBody);
		
		String sig = findMethodSignature (typeBody);
		sig = eliminateTypeArgs(sig);
		sig = normalizeWhitespace(sig).trim();
		
		String[] parts = sig.split(" ");
		
		return normalizeType (parts[parts.length - 1]);
	}
	
	public static String getShortHandMethodSig (String functionBody)
	{
		functionBody = eliminateDocs(functionBody);
		functionBody = eliminateComments(functionBody);
		
		String sig = findMethodSignature(functionBody);
		sig = eliminateTypeArgs(sig);
		sig = normalizeWhitespace(sig);
		
		int argsStartIndex = sig.lastIndexOf('(');	// might have annotations
		int argsEndIndex = sig.lastIndexOf(')');	// might have annotations
		
		String preArgs = sig.substring(0, argsStartIndex);
		String[] preArgTokens = preArgs.split(" ");
		StringBuilder b = new StringBuilder();
		String name = preArgTokens[preArgTokens.length - 1].trim();
		b.append(name);
		
		String argString = sig.substring(argsStartIndex + 1, argsEndIndex - 1);
		String[] args = argString.split(",");
		
		b.append("(");
		for (int i = 0; i < args.length; i++) {
			if (i != 0) {
				b.append(",");
			}
			String arg = args[i].trim();
			String[] parts = arg.split(" ");
			b.append(normalizeType(parts[0].trim()));
		}
		b.append(")");
		return b.toString();
	}
}
