package ca.mcgill.cs.swevo.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.Set;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static ca.mcgill.cs.swevo.util.UndirectedGraphUtil.*;

public class TestUndirectedGraphUtil {

	private final UndirectedGraph<String> graph = new UndirectedGraph<String>();
	
	@After
	public void setup () {
		graph.clear();
	}
	
	@Test
	public void testKnSingletonCase ()
	{
		graph.initNode("Hello");
		graph.initNode("Goodbye");
		
		Set<Set<String>> knSubgraphs = getMaximalKnSubgraphs2(graph);
		
		assertEquals (2, knSubgraphs.size());
	}
	
	@Test
	public void testSplitTwoSingletonsCase ()
	{
		graph.initNode("Hello");
		graph.initNode("Goodbye");
		
		assertEquals (2, getConnectedSubgraphs(graph).size());
	}
	
	@Test
	public void testGraphSplitSingletonCase ()
	{
		graph.initNode("Hello");
		
		Collection<UndirectedGraph<String>> connectedSubgraphs = getConnectedSubgraphs(graph);
		
		assertEquals (1, connectedSubgraphs.size());
	}
	
	@Test
	public void testGraphSplitThreeSingletonCase ()
	{
		graph.initNode("A");
		graph.initNode("B");
		graph.initNode("C");
		
		assertEquals(3, getConnectedSubgraphs(graph).size());
	}
	
	@Test
	public void testSingleK2Case ()
	{
		graph.addEdge("A", "B");
		
		Collection<Set<String>> knSubgraphs = getMaximalKnSubgraphs2(graph);
		
		assertEquals (1, knSubgraphs.size());
		
		Collection<String> first = CollectionUtil.getFirst(knSubgraphs);
		
		assertEquals(2, first.size());
	}
	
	@Test
	public void testDoubleK2Case ()
	{
		graph.addEdge("A", "B");
		graph.addEdge("C", "D");
		
		Collection<Set<String>> knSubgraphs = getMaximalKnSubgraphs2(graph);
		
		assertEquals(2, knSubgraphs.size());
		
		for (Collection<String> subGraph : knSubgraphs) {
			assertEquals (2, subGraph.size());
			if (subGraph.contains("A")) {
				assertTrue(subGraph.contains("B"));
			} else {
				assertTrue(subGraph.contains("C"));
				assertTrue(subGraph.contains("D"));
			}
		}
	}
	
	@Test
	public void testGraphSplitDoubleK2Case ()
	{
		graph.addEdge("A", "B");
		graph.addEdge("C", "D");
		
		assertEquals(2, getConnectedSubgraphs(graph).size());
	}
	
	@Test
	public void testSingleK3Case ()
	{
		graph.addEdges("A", "B", "C");
		
		Collection<Set<String>> knSubgraphs = getMaximalKnSubgraphs2(graph);
		
		assertEquals(1, knSubgraphs.size());
		
		Collection<String> first = CollectionUtil.getFirst(knSubgraphs);
		
		assertEquals(3, first.size());
		assertTrue(first.contains("A"));
		assertTrue(first.contains("B"));
		assertTrue(first.contains("C"));
	}
	
	@Test
	public void testSingleK15Case ()
	{
		graph.addEdges("1", "2", "3", "4", "5", "6", "7", "8", "9", "10","11","12", "13", "14", "15");
//					   "16", "17", "18", "19", "20");
		
		assertEquals(1, getMaximalKnSubgraphs2(graph).size());
	}
	
	@Test
	public void testConnLineGraph ()
	{
		graph.addEdge ("1", "2");
		graph.addEdge ("2", "3");
		graph.addEdge ("3", "4");
		
		Collection<?> connectedSubgraphs = getConnectedSubgraphs(graph);
		
		assertEquals(1, connectedSubgraphs.size());
	}
	
	@Test
	public void testKnLineGraph ()
	{
		graph.addEdge ("1", "2");
		graph.addEdge ("2", "3");
		graph.addEdge ("3", "4");
		
		assertEquals (3, getMaximalKnSubgraphs2(graph).size());
	}
	
	@Test
	public void testKnSquareGraph ()
	{
		graph.addEdge ("1", "2");
		graph.addEdge ("2", "3");
		graph.addEdge ("3", "4");
		graph.addEdge ("1", "4");
		
		assertEquals (4, getMaximalKnSubgraphs2(graph).size());
	}
	
	@Test
	public void testKnK32Graph ()
	{
		graph.addEdge ("1", "2");
		graph.addEdge ("2", "3");
		graph.addEdge ("3", "4");
		graph.addEdge ("1", "4");
		graph.addEdge ("2", "4");
		
		assertEquals (2, getMaximalKnSubgraphs2(graph).size());
	}
	
	@Test
	public void testKnLargeSparseGraph ()
	{
		// TODO cannot test larger yet, due to slowness
		for (int i = 1; i < 20; i++) {
			graph.addEdge ("" + i, "" + (i+1));
		}
		
		assertEquals(19, getMaximalKnSubgraphs2(graph).size());
	}
	
	@Test
	public void testAllEdgesUniqueness ()
	{
		graph.addEdge("1", "2");
		graph.addEdge("3", "4");
		graph.addEdge("2", "4");
		
		Collection<LRTuple<String>> edges = graph.getAllEdges();
		assertEquals (3, edges.size());
	}
}
